function TauPlot(setname,n)

fs = 16;
lw = 3;

if nargin < 2
   stats = TauBatch(setname);
else
   stats = TauBatch(setname,n);
   stats = stats(:,n);
end

% Collect runtimes, function values and solvers
tau    = zeros(1,size(stats,2));
labmda = zeros(1,size(stats,2));
T      = zeros(size(stats));
F      = zeros(size(stats));
Fp     = zeros(size(stats));
S      = cell(1,size(stats,1)); % Solver

for i=1:size(stats,1)
   S{i} = stats{i,1}.solver;
   for j=1:size(stats,2)
      T(i,j)  = stats{i,j}.cputime;
      F(i,j)  = stats{i,j}.f;
      Fp(i,j) = stats{i,j}.fPenalty;
      N(i,j)  = stats{i,j}.feval;

      tau(j)    = stats{1,j}.tau;   % [sic]
      lambda(j) = stats{1,j}.lambda;% [sic]
   end
end

figure(1); clf; gca; hold on;
idx = [1,2,4,5];
keys = {'BCD','SPG','PG','Smooth'};
style= {'b--','r-.','c-','k:'};
for i=1:length(idx)
   h = plot(tau,T(idx(i),:),style{i});
   set(h,'LineWidth',lw);
end
legend(keys{:}); box on;
set(gca,'YScale','log');
%set(gca,'Fontsize',fs);
xlabel('\tau'); ylabel('Runtime (seconds)');


figure(2); clf; gca; hold on;
idx = [1,5,4,2];
keys = {'BCD','Smooth','PG','SPG'};
style= {'r-.','k:','g--','b-'};
for i=1:length(idx)
   h = plot(tau,N(idx(i),:),style{i});
   set(h,'LineWidth',lw);
end
legend(keys{:},'location','northwest'); box on;
set(gca,'YScale','log');
set(gca,'Fontsize',fs);
set(gca,'XTick',[0,25,50,75,100]);
xlabel('\tau'); ylabel('Function evaluations');

% $$$ ylim([1e0,1e3]); ylabel('Runtime (seconds)');

% $$$ figure(2); clf;
% $$$ idx = [1,2,4,5];
% $$$ h = boxplot(N(idx,:)','labels',{'BGS','SPG','PG','Smooth'});
% $$$ set(gca,'YScale','log');
% $$$ ylim([1e-1,1e3]);

% $$$ figure(2);
% $$$  idx = [1,2,4,5];
% $$$  M = F(idx,:) - repmat(min(F(idx,:)),length(idx),1) + 1e-12;
% $$$  plot(M');
% $$$  set(gca,'YScale','log')
