function [] = TimeProjection2(nElements);
% Graph where we vary k

nTrials = 10;
kRange = 0:nElements/100:nElements;
type = 1;
param = 1;

for i = 1:length(kRange)
    k = ceil(kRange(i));
    k = max(1,min(k,nElements));
    
    fprintf('Running projection code on %d elements with k = %d\n',nElements,k);

    % randn
    for n = 1:nTrials
        x = generateData(type,nElements,param);
        
        % Compute Soft-Threshold value that will produce the appropriate
        % value of k
        sorted = sort(abs(x),'descend');
        p_k = sorted(k);
        LHS = sum(sorted(1:k-1)) - (k-1)*p_k;
        if k < nElements
            p_kP1 = sorted(k+1);
        else
            p_kP1 = 0;
        end
        ST = (p_k+p_kP1)/2;
        tau = LHS - k*(ST-p_k);
        
        t = cputime;
        projectRandom2C(abs(x),tau);
        runTime(i,n,1) = cputime-t;
        t = cputime;
        projectHeapC(abs(x),tau);
        runTime(i,n,2) = cputime-t;
        t = cputime;
        p=projectSortC(abs(x),tau);
        runTime(i,n,3) = cputime-t;
        
        drawnow
        
    end

end

% Store the results
[pathstr,name,ext,versn] = fileparts(mfilename('fullpath'));
filename = sprintf('%s%sdata%stimeProjection2_%d.mat',...
                   pathstr,filesep,filesep,nElements);
save(filename,'kRange', 'nElements','runTime');
