function [] = TimeProjectionPlot1(kRatio)

doTitle = 0;
fs      = 12; % Font size;

% Load data
[pathstr,name,ext,versn] = fileparts(mfilename('fullpath'));
datafile = sprintf('%s%sdata%stimeProjection1_%.2f.mat',...
                   pathstr,filesep,filesep,kRatio);

if ~exist(datafile,'file')
   fprintf('Please run TimeProjection1(%f) first.\n',kRatio);
   return;
else
   load(datafile);
end



figure('Units', 'pixels', ...
    'Position', [100 100 500 375]);
clf;hold on;

doMedian = 0;
for i = 1:3
    RT = runTime(:,:,i);

    randomMean = mean(RT,2);
    randomMedian = median(RT,2);
    randomLower = min(RT,[],2);
    randomUpper = max(RT,[],2);

    hLower(i) = line(elementRange,randomLower);
    hUpper(i) = line(elementRange,randomUpper);
    hMean(i) = line(elementRange,randomMean);
    if doMedian
        hMedian(i) = line(elementRange,randomMedian);
    end
    %hE(i) = errorbar(elementRange,randomMedian,randomLower,randomUpper);
end

% Truncate plot to non-negative values
xl = xlim;
xl(1) = 0;
yl = ylim;
yl(1) = 0;
xlim(xl);
ylim(yl);

set(hLower(1)                          , ...
    'LineStyle'       , '-'        , ...
    'Color'           , [.75 .75 1] , ...
    'LineWidth'       , 1.5         );
set(hUpper(1)                          , ...
    'LineStyle'       , '-'        , ...
    'Color'           , [.75 .75 1] , ...
    'LineWidth'       , 1.5         );
set(hMean(1)                          , ...
    'Color'           , [0 0 .5]    , ...
     'LineStyle'       , '-'        , ...
    'LineWidth'       , 2           );
if doMedian
    set(hMedian(1)                           , ...
        'LineStyle'       , 'none'      , ...
        'LineWidth'       , 1           , ...
        'Marker'          , 'o'         , ...
        'MarkerSize'      , 6           , ...
        'MarkerEdgeColor' , [.2 .2 .2]  , ...
        'MarkerFaceColor' , [.75 .75 1]  , ...
        'Color'           , [.3 .3 .3]  );
end

set(hLower(2)                          , ...
    'LineStyle'       , '--'        , ...
    'Color'           , [.75 1 .75] , ...
    'LineWidth'       , 1.5         );
set(hUpper(2)                          , ...
    'LineStyle'       , '--'        , ...
    'Color'           , [.75 1 .75] , ...
    'LineWidth'       , 1.5         );
set(hMean(2)                          , ...
    'Color'           , [0 .5 0]    , ...
     'LineStyle'       , '--'        , ...
    'LineWidth'       , 2           );
if doMedian
    set(hMedian(2)                           , ...
        'LineStyle'       , 'none'      , ...
        'LineWidth'       , 1           , ...
        'Marker'          , 'o'         , ...
        'MarkerSize'      , 6           , ...
        'MarkerEdgeColor' , [.2 .2 .2]  , ...
        'MarkerFaceColor' , [.75 1 .75]  , ...
        'Color'           , [.3 .3 .3]  );
end

set(hLower(3)                          , ...
    'LineStyle'       , '-.'        , ...
    'Color'           , [1 .75 .75] , ...
    'LineWidth'       , 1.5         );
set(hUpper(3)                          , ...
    'LineStyle'       , '-.'        , ...
    'Color'           , [1 .75 .75] , ...
    'LineWidth'       , 1.5         );
set(hMean(3)                          , ...
    'Color'           , [.5 0 0]    , ...
     'LineStyle'       , '-.'        , ...
    'LineWidth'       , 2           );
if doMedian
    set(hMedian(3)                           , ...
        'LineStyle'       , 'none'      , ...
        'LineWidth'       , 1           , ...
        'Marker'          , 'o'         , ...
        'MarkerSize'      , 6           , ...
        'MarkerEdgeColor' , [.2 .2 .2]  , ...
        'MarkerFaceColor' , [1 .75 .75]  , ...
        'Color'           , [.3 .3 .3]  );
end

if doTitle
    hTitle = title(sprintf('Projection Time against Vector Length (Sparsity = %d%%)',kRatio*100));
end
hXLabel = xlabel('Vector Length');
hYLabel = ylabel('Runtime (s)');

hLegend = legend([hMean(1),hMean(2),hMean(3)],...
    'Random',...
    'Heap',...
    'Sort','Location','NorthWest');

%set( gca                       , ...
%    'FontName'   , 'Helvetica' );
set([hLegend, gca]             , ...
    'FontSize'   , 10           );
set([hXLabel, hYLabel]  , ...
    'FontSize'   , fs          );
if doTitle
%    set([hTitle, hXLabel, hYLabel], ...
%        'FontName'   , 'AvantGarde');
    set( hTitle                    , ...
        'FontSize'   , fs          , ...
        'FontWeight' , 'bold'      );
else

%    set([hXLabel, hYLabel], ...
%        'FontName'   , 'AvantGarde');
end

set(gca, ...
    'Box'         , 'off' ,...
    'TickDir'     , 'out'     , ...
    'TickLength'  , [.02 .02] , ...
    'XMinorTick'  , 'on'      , ...
    'YMinorTick'  , 'on'      , ...
    'XColor'      , [.0 .0 .0], ...
    'YColor'      , [.0 .0 .0], ...
    'LineWidth'   , 1         );

set(gcf, 'PaperPositionMode', 'auto');

