clear all
close all

%% Prepare a 3D Multi-Class noisy version of 'X.png'

nInstances = 2; % Number of examples to generate
nSlices = 1; % Number of 2D slices to stack up to make 3D volume

% load state.mat
% randn('state',state);
state = randn('state');

% Load image/label data
label = sign(double(imread('misc/X.png'))-1);
label = label(:,:,1);

% Convert from {-1,1} to {1,2} representation
y = label;
y(y==-1) = 2;

[nRows nCols] = size(label);
nFeatures = 3; % Red-Green-Blue Features

% Multi-Class: we will divide the X into 3 parts,
% based on the its quadrant in the image
tmp = y(1:16,17:32,:);
tmp(tmp==2) = 3;
y(1:16,17:32,:) = tmp;
tmp = y(17:32,1:16,:);
tmp(tmp==2) = 4;
y(17:32,1:16,:) = tmp;
tmp = y(17:32,17:32,:);
tmp(tmp==2) = 4;
y(17:32,17:32,:) = tmp;
nStates = 4; % Parts will be 2:4, background will be 1

% Show the multi-class X
subplot(1,2,1);
imshow(y,[]);
title('Multi-Class Label');

% The features will be the color channels
colormapping = [1 1 1;1 0 0;0 1 0;0 0 1];
Xred = zeros(nRows,nCols);
Xred(:) = y(:) == 1 | y(:)==2;
Xgreen = zeros(nRows,nCols);
Xgreen(:) = y(:) == 1 | y(:) == 3;
Xblue = zeros(nRows,nCols);
Xblue(:) = y(:) == 1 | y(:) == 4;
clear X
I(:,:,1) = Xred;
I(:,:,2) = Xgreen;
I(:,:,3) = Xblue;
subplot(1,2,2);
imshow(I);
title('Multi-Class RGB Features');

% 3D: we will stack up the multi-color X to make
% a 3D volume with nSlices slices
nNodes = nRows*nCols*nSlices;

y = repmat(y,[1 1 nSlices]);
y = reshape(y,[1 nNodes]);
y = repmat(y,[nInstances 1]);

figure(2);
for i = 1:nInstances
    for s = 1:nSlices
        Xslice = I + randn(size(I))/2;
        if s <= 4
            subplot(2,2,s);
            imshow(Xslice);
            title(sprintf('Slice %d of Noisy Observation\n',s));
        end

        % Red
        red(:,:,s) = Xslice(:,:,1);
        green(:,:,s) = Xslice(:,:,2);
        blue(:,:,s) = Xslice(:,:,3);
    end
    X(i,1,:) = reshape(red,[1 1 nRows*nCols*nSlices]);
    X(i,2,:) = reshape(green,[1 1 nRows*nCols*nSlices]);
    X(i,3,:) = reshape(blue,[1 1 nRows*nCols*nSlices]);
end


%%
tied = 1;
ising = 1;
useMex = 1;
maxIter = 50; % Number of iterations for LBP

% Standardize Features
fprintf('Standardizing Features\n');
X = UGM_standardizeCols(X,tied);

% Make 3D adjacency matrix
fprintf('Forming 3D Lattice Adjacency Matrix\n');
    adjMatrix = fixed_Lattice3D(nRows,nCols,nSlices);
    
% Make edgeStruct from adjacency matrix
fprintf('Making edgeStruct\n');
tic
edgeStruct = UGM_makeEdgeStruct(adjMatrix);
nEdges = edgeStruct.nEdges;
toc

% Make non-negative edge features
fprintf('Making non-negative edge features\n');
Xedge = UGM_makeEdgeFeaturesInvAbsDif(X,edgeStruct.edgeEnds);

% Add Bias
fprintf('Adding Node/Edge bias\n');
X = [ones(nInstances,1,nNodes) X];
Xedge = [ones(nInstances,1,nEdges) Xedge];

% Make Infostruct
fprintf('Making infoStruct\n');
infoStruct = UGM_makeInfoStruct(X,Xedge,edgeStruct,nStates,ising,tied,useMex,maxIter);

% Inititalize Weights
fprintf('Initializing Weights\n');
[w,v] = UGM_initWeights(infoStruct,'random');

%% Example of Decoding

% Make potentials for first example with initial weights
fprintf('Making Node Potentials\n');
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
fprintf('Making Edge Potentials\n');
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding with initial weights
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
y_ICM = reshape(y_ICM,[nRows nCols nSlices]);
figure(3);
for s = 1:min(4,nSlices)
    subplot(2,2,s);
    imshow(y_ICM(:,:,s),[1 nStates]);
    title(sprintf('ICM Decoding of Slice %d (rand params)',s));
end
colormap(colormapping);

%% Example of cross-validation and training using UGM_train

options.maxFunEvals = 50;
options.nonNegEdge = 0; % Enforce that edge weights be non-negative
options.nodeRegType = 2; % L2-Regularization on Nodes
options.nodeRegStr = 2.^[4:-1:-4]; % Node Regularizer Strengths to test
options.edgeRegType = 1; % L2-Regularization on Edges
options.edgeRegStr = 2.^[4:-1:-4]; % Edge Regularizer Strength to test
[w,v] = UGM_train(@UGM_PseudoLoss,options,X,Xedge,y,edgeStruct,infoStruct);

% To do Loopy Training:
%[w,v] = UGM_train(@UGM_Loss,options,X,Xedge,y,edgeStruct,infoStruct,@UGM_Infer_LBP);


%%

% Make potentials for first example with learned weights
nodePot = UGM_makeNodePotentials(X(1,:,:),w,infoStruct);
edgePot = UGM_makeEdgePotentials(Xedge(1,:,:),v,edgeStruct,infoStruct);

% Do ICM Decoding with Learned weights
fprintf('Decoding w/ ICM\n');
y_ICM = UGM_Decode_ICM(nodePot,edgePot,edgeStruct,infoStruct);
y_ICM = reshape(y_ICM,[nRows nCols nSlices]);
figure(4);
for s = 1:min(4,nSlices)
    subplot(2,2,s);
    imshow(y_ICM(:,:,s),[1 nStates]);
    title(sprintf('ICM Decoding of Slice %d (learned params)',s));
end
colormap(colormapping);
pause(1);

% Do LBP Decoding with Learned weights
if 0
    fprintf('Decoding w/ LBP (needs to be mexed)\n');
    y_ICM = UGM_Decode_LBP(nodePot,edgePot,edgeStruct,infoStruct);
    y_ICM = reshape(y_ICM,[nRows nCols nSlices]);
    figure(5);
    for s = 1:min(4,nSlices)
        subplot(2,2,s);
        imshow(y_ICM(:,:,s),[1 nStates]);
        title(sprintf('LBP Decoding of Slice %d (learned params)',s));
    end
    colormap(colormapping);
end
